# personal modules
import segmodels
import segdata
import mcmc
from metrowrapper import MetroWrapper

# standalone file
from file_list import l

# standard modules
import numpy as np
import math
import matplotlib.pyplot as plt
import tensorflow as tf
# import memory_profiler
import os.path
import datetime
import pickle

# remove tf log messages:
import os
os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"


if __name__ == '__main__':

    # test on the first 20 h of segregostat (l[0]) and the first 30h of
    # chemostat (l[2])
    data=np.array([segdata.Data(l[0])])
    for d in data:
        d.sanitize()
    data[0].truncate(40)


    # memory:
    #             time,Ca,Cg, p, dp
    mem=np.array([[-5, 0, .01, 0, 0],
                  [ 0, 0, .01, 0, 0]])

    memo=segmodels.Memory(memory=mem)

    f="results/metropolis_2021-05-10_segregostat_array"

    # run of all the possibilities once

    n=3

    if os.path.exists(f+"tmp"+".pkl"):
        with open(f+"tmp"+'.pkl', "rb") as input:
            i=0
            while True:
                try:
                    list_metro=pickle.load(input)
                    i+=1
                except EOFError:
                    break
            input.close()
    elif os.path.exists(f+".pkl"):
        with open(f+'.pkl', "rb") as input:
            i=0
            while True:
                try:
                    list_metro=pickle.load(input)
                    i+=1
                except EOFError:
                    break
            input.close()
    else:
        list_metro=np.empty(n*20,dtype="object")
        list_metro.fill(None)


    print(list_metro)

    # ignore the ones with full switching, unpractical
    # possibilities up till 24 then
    # from 4 because before that no sitching to ara at all
    for i in range(4,24):
        for j in range(n):
            if list_metro[j*20+i-4] is None:
                if 'met' not in locals():
                    diff, temp_memo=segmodels.differential.set_contri(memo.clone(), i)
                    mod=segmodels.Model(temp_memo, diff)
                    wrapper=MetroWrapper(mod, data, beta=1000)
                    met=mcmc.MetropolisHastings(wrapper)
                metro=met.clone()
                list_metro[j*20+i-4]=metro
        if 'met' in locals():
            del met

    del data
    del memo
    del mem
    if 'diff' in locals():
        del diff
    if 'temp_memo' in locals():
        del temp_memo
    if 'wrapper' in locals():
        del wrapper
    del j

    # for i in list_names:print(i)

    # for iter in range(2,3,2):
    for iter in range(101,802,100):
        for i,met in enumerate(list_metro):
            if met.aggregate.shape[0]<iter:
                print("######################################")
                print(i%20+4)
                print(i//20)
                print("######################################")
                met.run(iter-met.aggregate.shape[0])
                list_metro[i].mod=list_metro[i].mod.clone(full=False)
                with open(f+'tmp'+'.pkl', 'wb') as output:
                    pickle.dump(list_metro, output, pickle.HIGHEST_PROTOCOL)
                    output.close()
        if os.path.exists(f+'tmp'+'.pkl'):
            if os.path.exists(f+".pkl"):
                os.remove(f+".pkl")
            os.rename(f+'tmp'+'.pkl',f+".pkl")
