from mcmc import MetropolisHastings
import segmodels
import segdata
import matplotlib.pyplot as plt
import numpy as np
import pickle

# remove tf log messages:
import os
os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"


if __name__ == '__main__':

    ## READ DATA

    f="results/metropolis_2021-05-14_array" # ran previously
    # Chemostat
    # f="results/metropolis_2021-05-29_chemostat_array" # ran on 2021-05-
    # segregostat
    # f="results/metropolis_2021-05-28_segregostat_array" # ran on 2021-05-31

    metro=MetropolisHastings.read(f)

    min_num=3
    mods=np.empty((min_num, 3), dtype=object)
    mean_mods=np.empty((4,), dtype=object)

    ## create array to save min models later
    if not os.path.exists(f+'/save_'+str(min_num)+'_models.pkl'):
        print(f+'/save_'+str(min_num)+'_models.pkl')
        to_save=np.empty((20,(min_num+1)*3+1), dtype=object)
        to_save.fill(None)
    else:
        with open(f+'/save_'+str(min_num)+'_models.pkl', 'rb') as input:
            to_save=pickle.load(input)
            input.close()

    for j in range(20):
        # j+4 is the model number
        t='_'.join(segmodels.differential.get_contri(j+4))
        t=t.replace(' ','')
        for i in range(0,60, 20):
            # make the graphs for one metro algo
            metro[i+j].show(pre=t)
            # get the min models
            mods[:,i//20]=metro[i+j].get_mins(min_num)

        ## SAVE PLOTS METROPOLIS

        # # uncomment to save figures for the metropolis evolution
        for p, para in enumerate(metro[j].aggregate[0,:-1]):
            if not np.isnan(para):
                if not os.path.exists(f):
                    os.mkdir(f)
                name=f+'/'+str(j+4)+'_'+t+'_'+metro[j].mod.info_num(p)[0]+'.pdf'
                name=name.replace(" ", "")
                plt.figure(p)
                plt.savefig(name, bbox_inches='tight')
                plt.clf()

        ## TREAT MIN MODELS

        if to_save[j,0] is None:
            # min_num min models
            # mean( min models) for each metropolis run
            # mean(mean(min models)) mean of the mean models between runs

            agg=np.empty((min_num,mods[0,0].aggregate().shape[0]))
            mean_agg=np.empty((mods[0,0].aggregate().shape[0],))
            mean_agg[0]=np.nan

            for k in range(mean_mods.shape[0]-1):
                # make the mean of the models
                mean_mods[k]=mods[0,k].clone() # initial model

                for l,m in enumerate(mods[:,k]):
                    # run model once for easy use later
                    print("min model")
                    print(j+4)
                    print(k)
                    print(l)
                    m.run_once()
                    print("done")
                    agg[l,:]=m.aggregate()

                for n in range(1,agg.shape[1]):
                    _, lin=m.info_num(n)
                    if lin:
                        mean_agg[n]=np.mean(agg[:,n])
                    else:
                        mean_agg[n]=np.exp(np.mean(np.log(agg[:,n])))
                mean_mods[k].degregate(mean_agg)
                print("mean min model")
                print(j+4)
                print(k)
                mean_mods[k].run_once()
                print("done")
                mean_mods[k].log_evaluate()

            mean_mods[-1]=mean_mods[0].clone()
            agg=np.empty((3,mods[0,0].aggregate().shape[0]))
            for l,m in enumerate(mean_mods[:-1]):
                # run model once for easy use later
                m.run_once()
                agg[l,:]=m.aggregate()
            for n in range(1,agg.shape[1]):
                _, lin=m.info_num(n)
                if lin:
                    mean_agg[n]=np.mean(agg[:,n])
                else:
                    mean_agg[n]=np.exp(np.mean(np.log(agg[:,n])))
            mean_mods[-1].degregate(mean_agg)
            print("min model global")
            m.run_once()
            print("done")
            mean_mods[-1].run_once()
            mean_mods[-1].log_evaluate()

            ## SAVE MIN MODELS FOR LATER

            for i in range(mods.shape[1]):
                to_save[j,i*min_num:(i+1)*min_num]=mods[:,i]
            to_save[j,-min_num-1:]=mean_mods
            with open(f+'/save_'+str(min_num)+'_models.pkl', 'wb') as output:
                print(to_save)
                pickle.dump(to_save, output, pickle.HIGHEST_PROTOCOL)
                output.close()
        else:
            for i in range(mods.shape[1]):
                mods[:,i]=to_save[j,i*min_num:(i+1)*min_num]
            mean_mods=to_save[j,-min_num-1:]

        ## PLOT MIN MODELS

        # first print all final data for consistency check

        for k in range(mods.shape[1]):
            print(k)
            for l in range(mods.shape[0]):
                print(l)
                print(mods[l,k].aggregate())
            print("aggregate")
            print(mean_mods[k].aggregate())
        print("final")
        print(mean_mods[-1].aggregate())

        fig_num=100
        data=mean_mods[0].data
        for i, d in enumerate(data):
            fig, ax = plt.subplots(num=fig_num+i)
            p=segdata.Plot(d)
            # 1) plot data
            p.plot_count(ax, d)

            # 2) plot all the graphs
            for row in mods:
                for model in row:
                    model.models[i].plot_prop(ax)

            for k,model in enumerate(mean_mods):
                model.models[i].plot_prop(ax, label='mean '+str(k))

            # 3) finishing touch

            p.add_regulation(ax, d)
            fig.suptitle(t)
            name=f+'/'+str(j+4)+'_'+t+'_min_models_'+str(i)+'.pdf'
            plt.savefig(name, bbox_inches='tight')
            plt.clf()
