from mcmc import MetropolisHastings
import segmodels
import segdata
import matplotlib.pyplot as plt
import numpy as np
import pickle
import pandas as pd

from file_list import l

# remove tf log messages:
import os
os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"


if __name__ == '__main__':

    ## READ DATA

    # temp
    # f="results/metropolis_2021-05-10_chemostat_arraytmp"

    # final
    f="results/metropolis_2021-05-14_array"# ran with corrected axes names
    # Chemostat
    # f="results/metropolis_2021-05-29_chemostat_array" # ran with corrected axes names
    # segregostat
    # f="results/metropolis_2021-05-28_segregostat_array" # ran with corrected axes names
    min_num=3

    ## create array to save min models later
    with open(f+'/save_'+str(min_num)+'_models.pkl', 'rb') as input:
        best_models=pickle.load(input)
        input.close()
        del input

    # best_models:
    # lines: 0->20 corresponds to models 4->23
    # columns: 3 min models for each 3 runs then 3 mean(min models) per run
    #          then mean(mean(min models)) for all runs.


    ## eliminate a few models that had errors
    ## (errors appeared at first then the code was corrected)
    # for f="results/metropolis_2021-05-14_array"
    best_models[9,3]=best_models[9,2]
    best_models[9,4]=best_models[9,2]
    best_models[8,3]=best_models[8,2]
    best_models[8,4]=best_models[8,2]
    best_models[8,5]=best_models[8,2]
    best_models[8,10]=best_models[8,2]

    # # for f="results/metropolis_2021-05-10_chemostat_arraytmp"
    # # None

    ## get the min models per line
    min_models=np.empty((20,), dtype=object)
    for i in range(20):
        min_models[i]=best_models[i,0]
        for mod in best_models[i,1:]:
            if mod.aggregate()[0]<min_models[i].aggregate()[0]:
                min_models[i]=mod

    del best_models

    ## PLOT MIN MODELS



    # first print all final data for consistency check
    # save to file too
    to_save=np.zeros((20,min_models[0].aggregate().shape[0]+1))
    indexes=[]
    for i,m in enumerate(min_models):
        agg=m.aggregate()
        print(agg)
        to_save[i,1:]=agg
        to_save[i,0]=i+4
        if i+4>=16:
            to_save[i,-1]=to_save[i,12]
            to_save[i,12]=np.nan
        else:
            to_save[i,-1]=np.nan
        indexes.append('_'.join(segmodels.differential.get_contri(i+4)))

    df=pd.DataFrame(to_save)
    df.columns=[
        'Model number',
        '$\log(\cosh())$',
        '$\Delta t_g$',
        '$\Delta t_a$',
        '$K_g$',
        '$K_a$',
        '$\mu_{g, \max}$',
        '$\mu_{a, \max}$',
        'threshold or $k$',
        'dtf',
        '$\Delta t_i$',
        '$r_a$ or $r$',
        'rd',
        '$H_1$',
        '$H_2$',
        '$Y_{x,a}$',
        '$Y_{x,g}$',
        'Initial reactive portion',
        '$n$',
    ]
    df.index=indexes
    df.to_excel(f+'/'+"global_base.xlsx")



    # make graphs for the ultimate best model
    fig_num=100
    data=min_models[0].data
    for j, mod in enumerate(min_models):
        t='_'.join(segmodels.differential.get_contri(j+4))
        t=t.replace(' ','')
        for i, d in enumerate(data):
            fig, ax = plt.subplots(num=fig_num+i*20+j,figsize=(4,4))
            p=segdata.Plot(d)
            # 1) plot data
            p.plot_count(ax, d)

            # 2) plot all the graphs

            mod.models[i].plot_prop(ax)

            # 3) finishing touch

            p.add_regulation(ax, d)
            name=f+'/'+str(j+4)+'_'+t+'_absolute_min_'+str(i)+'.pdf'
            plt.savefig(name, bbox_inches='tight')
            plt.clf()

            #####

            # plot evolution of concentrations
            fig, ax = plt.subplots(num=fig_num+i*20+j,figsize=(4,4))
            p=segdata.Plot(d)
            # 1) plot data
            # no reference for concentrations
            # 2) plot all the graphs

            plt.plot(mod.models[i].sol[0,:],np.sum(mod.models[i].sol[1:4,:],
                    axis=0),
                    label='Cell concentration')

            plt.plot(mod.models[i].sol[0,:],mod.models[i].sol[2,:],
                    label='Arabinose user concentration')

            plt.plot(mod.models[i].sol[0,:],
                    mod.models[i].sol[1,:]+mod.models[i].sol[3,:],
                    label='Glucose user concentration')

            # 3) finishing touch

            p.add_regulation(ax, d)
            ax.legend()
            ax.set_xlabel('Time [h]')
            ax.set_ylabel('Concentration [g/L]')
            name=f+'/'+str(j+4)+'_'+t+'_absolute_min_cell_cc_'+str(i)+'.pdf'
            plt.savefig(name, bbox_inches='tight')
            plt.clf()


            # plot evolution of cell concentrations -- no legend
            fig, ax = plt.subplots(num=fig_num+i*20+j,figsize=(4,4))
            p=segdata.Plot(d)
            # 1) plot data
            # no reference for concentrations
            # 2) plot all the graphs

            plt.plot(mod.models[i].sol[0,:],np.sum(mod.models[i].sol[1:4,:],
                    axis=0),
                    label='Cell concentration')

            plt.plot(mod.models[i].sol[0,:],mod.models[i].sol[2,:],
                    label='Arabinose user concentration')

            plt.plot(mod.models[i].sol[0,:],
                    mod.models[i].sol[1,:]+mod.models[i].sol[3,:],
                    label='Glucose user concentration')

            # 3) finishing touch

            p.add_regulation(ax, d)
            # ax.legend()
            ax.set_xlabel('Time [h]')
            ax.set_ylabel('Concentration [g/L]')
            name=f+'/'+str(j+4)+'_'+t+'_absolute_min_cell_cc_no_legend_'+str(i)+'.pdf'
            plt.savefig(name, bbox_inches='tight')
            plt.clf()


            #####

            # plot evolution of sugar concentrations
            fig, ax = plt.subplots(num=fig_num+i*20+j,figsize=(4,4))
            p=segdata.Plot(d)
            # 1) plot data
            # no reference for concentrations
            # 2) plot all the graphs


            plt.plot(mod.models[i].sol[0,:],mod.models[i].sol[4,:],
                    label='Arabinose concentration')

            plt.plot(mod.models[i].sol[0,:],
                     mod.models[i].sol[5,:],
                    label='Glucose concentration')

            # 3) finishing touch

            p.add_regulation(ax, d)
            ax.legend()
            ax.set_xlabel('Time [h]')
            ax.set_ylabel('Concentration [g/L]')
            name=f+'/'+str(j+4)+'_'+t+'_absolute_min_sugar_cc_'+str(i)+'.pdf'
            plt.savefig(name, bbox_inches='tight')
            plt.clf()

    del data
    del df
    plt.close('all')

    # # make graphs to use the models on different data

    fig_num=1
    d=segdata.Data(l[3])
    d.sanitize()

    to_save=np.zeros((20,min_models[0].aggregate().shape[0]+1))

    for j, mod in enumerate(min_models):
        if j+4!=9 and j+4!=11:
            t='_'.join(segmodels.differential.get_contri(j+4))
            t=t.replace(' ','')

            fig, ax = plt.subplots(num=fig_num+j,figsize=(8,4))
            p=segdata.Plot(d)
            # 1) plot data
            p.plot_count(ax, d)

            # 2) plot all the graphs

            cop=mod.clone(full=False)
            cop.data=np.array([d])
            cop.run_once()
            agg=cop.aggregate()
            print(agg)
            to_save[j,1:]=agg
            to_save[j,0]=j+4
            if i+4>=16:
                to_save[j,-1]=to_save[j,12]
                to_save[j,12]=np.nan
            else:
                to_save[j,-1]=np.nan

            cop.models[0].plot_prop(ax)

            # 3) finishing touch

            p.add_regulation(ax, d)
            name=f+'/'+str(j+4)+'_'+t+'_absolute_min_alt_'+str(i)+'.pdf'
            plt.savefig(name, bbox_inches='tight')
            plt.clf()
            plt.close('all')
        else:
            to_save[j,:]=np.nan

    # save alternative data as excel

    df=pd.DataFrame(to_save)
    df.columns=[
        'Model number',
        '$\log(\cosh())$',
        '$\Delta t_g$',
        '$\Delta t_a$',
        '$K_g$',
        '$K_a$',
        '$\mu_{g, \max}$',
        '$\mu_{a, \max}$',
        'threshold or $k$',
        'dtf',
        '$\Delta t_i$',
        '$r_a$ or $r$',
        'rd',
        '$H_1$',
        '$H_2$',
        '$Y_{x,a}$',
        '$Y_{x,g}$',
        'Initial reactive portion',
        '$n$',
    ]
    df.index=indexes
    df.to_excel(f+'/'+"global_alt.xlsx")

    del to_save

    # make graphs segregostat for the chemostat optimisation
    fig_num=200
    i=0
    if "chemostat" in f:
        d=segdata.Data(l[0])
        d.sanitize()

        to_save=np.zeros((20,min_models[0].aggregate().shape[0]+1))

        for j, mod in enumerate(min_models):
            if j+4!=9 and j+4!=11:
                t='_'.join(segmodels.differential.get_contri(j+4))
                t=t.replace(' ','')

                fig, ax = plt.subplots(num=fig_num+j,figsize=(8,4))
                p=segdata.Plot(d)
                # 1) plot data
                p.plot_count(ax, d)

                # 2) plot all the graphs

                cop=mod.clone(full=False)
                cop.data=np.array([d])
                cop.run_once()
                cop.aggregate()
                cop.models[0].plot_prop(ax)

                # 3) finishing touch

                p.add_regulation(ax, d)
                name=f+'/'+str(j+4)+'_'+t+'_absolute_min_seg_'+str(i)+'.pdf'
                plt.savefig(name, bbox_inches='tight')
                plt.clf()

                #####

                # plot evolution of concentrations
                fig, ax = plt.subplots(num=fig_num+i*20+j,figsize=(4,4))
                p=segdata.Plot(d)
                # 1) plot data
                # no reference for concentrations
                # 2) plot all the graphs

                plt.plot(cop.models[i].sol[0,:],np.sum(cop.models[i].sol[1:4,:],
                        axis=0),
                        label='Cell concentration')

                plt.plot(cop.models[i].sol[0,:],cop.models[i].sol[2,:],
                        label='Arabinose user concentration')

                plt.plot(cop.models[i].sol[0,:],
                        cop.models[i].sol[1,:]+cop.models[i].sol[3,:],
                        label='Glucose user concentration')

                # 3) finishing touch

                p.add_regulation(ax, d)
                ax.legend()
                ax.set_xlabel('Time [h]')
                ax.set_ylabel('Concentration [g/L]')
                name=f+'/'+str(j+4)+'_'+t+'_absolute_min_seg_cell_cc_'+str(i)+'.pdf'
                plt.savefig(name, bbox_inches='tight')
                plt.clf()


                # plot evolution of cell concentrations -- no legend
                fig, ax = plt.subplots(num=fig_num+i*20+j,figsize=(4,4))
                p=segdata.Plot(d)
                # 1) plot data
                # no reference for concentrations
                # 2) plot all the graphs

                plt.plot(cop.models[i].sol[0,:],np.sum(cop.models[i].sol[1:4,:],
                        axis=0),
                        label='Cell concentration')

                plt.plot(cop.models[i].sol[0,:],cop.models[i].sol[2,:],
                        label='Arabinose user concentration')

                plt.plot(cop.models[i].sol[0,:],
                        cop.models[i].sol[1,:]+cop.models[i].sol[3,:],
                        label='Glucose user concentration')

                # 3) finishing touch

                p.add_regulation(ax, d)
                # ax.legend()
                ax.set_xlabel('Time [h]')
                ax.set_ylabel('Concentration [g/L]')
                name=f+'/'+str(j+4)+'_'+t+'_absolute_min_seg_cell_cc_no_legend_'+str(i)+'.pdf'
                plt.savefig(name, bbox_inches='tight')
                plt.clf()


                #####

                # plot evolution of sugar concentrations
                fig, ax = plt.subplots(num=fig_num+i*20+j,figsize=(4,4))
                p=segdata.Plot(d)
                # 1) plot data
                # no reference for concentrations
                # 2) plot all the graphs


                plt.plot(cop.models[i].sol[0,:],cop.models[i].sol[4,:],
                        label='Arabinose concentration')

                plt.plot(cop.models[i].sol[0,:],cop.models[i].sol[5,:],
                        label='Glucose concentration')

                # 3) finishing touch

                p.add_regulation(ax, d)
                ax.legend()
                ax.set_xlabel('Time [h]')
                ax.set_ylabel('Concentration [g/L]')
                name=f+'/'+str(j+4)+'_'+t+'_absolute_min_seg_sugar_cc_'+str(i)+'.pdf'
                plt.savefig(name, bbox_inches='tight')
                plt.clf()
