import numpy as np
from . import contributions as ctrb
import math
import copy

contrib={
    "dilution":ctrb.dilution,
    "switching":ctrb.switching,
    "growth":ctrb.growth,
    "react_mid":ctrb.react_mid,
    "react_up":ctrb.react_up,
    "react_down":ctrb.react_down,
    # follow_p should always be after the react contributions
    "follow_p":ctrb.follow_p
}

# set contri_detection is set that way as there are 4 possible states for
# the detection:
# * the full thing that follows p
# * only the initial full change, and then no more
# * none at all
# * an alternate switching strategy
# _set_contri function take one Differential object, one Memory object, and
# one boolean
# they return the Differential and the Memory
def _set_contri_consumption(diff, mem, s):
    """ if s is True, there is co-consumption"""
    # nothing to change in memory, all parameters are the same
    if s:
        diff.func["growth"]=ctrb.growth
    else:
        diff.func["growth"]=ctrb.single_growth

def _get_contri_consumption(defi, s):
    if s:
        c='co-consume'
    else:
        c='ara only  '
    defi.append(c)

def _set_contri_delay(diff, mem, s):
    """ if s is True, there is a delay"""
    # dta and dtg need to get to 0 in memory
    # no change in the differential
    if not s:
        mem.dtg=np.nan
        mem.dta=np.nan

def _get_contri_delay(defi, s):
    if s:
        c='delay   '
    else:
        c='no delay'
    defi.append(c)

def _set_contri_switching(diff, mem, s):
    """if s then there is constant switching"""
    if s:
        diff.func["switching"]=ctrb.switching
    else:
        if "switching" in diff.func:
            diff.func.pop("switching")
        mem.H1=np.nan
        mem.H2=np.nan

def _get_contri_switching(defi, s):
    if s:
        c='switch   '
    else:
        c='no switch'
    defi.append(c)

def _set_contri_detection_1(diff, mem, s):
    """ if s is 1: complicated switching scheme"""
    if not s:
        if "react_up" in diff.func:
            diff.func.pop("react_up")
        if "react_down" in diff.func:
            diff.func.pop("react_down")
        if "react_mid" in diff.func:
            diff.func.pop("react_mid")
        if "follow_p" in diff.func:
            diff.func.pop("follow_p")
        mem.dtf=np.nan
        mem.dti=np.nan
        mem.rd=np.nan
        mem.init_reactive=np.nan
        mem.memory[np.logical_not(np.isnan(mem.memory[:,3])),3]=1

def _get_contri_detection_1(defi, s):
    if s:
        c='1'
    else:
        c='0'
    defi.append(c)

def _set_contri_detection_2(diff, mem, s):
    """
    if s is one:
        if there is no switching:
            add alternate switching scheme
    if s is false:
        if there is a switching scheme:
            remove react_down
    """
    if s:
        if not "react_up" in diff.func:
            diff.func["alt_react"]=ctrb.alt_react
            # use the unused parameters that vary the right way
            # thresh should be left
            # ra should be left
            # rd is used instead of the exponent as it varies the same way
            mem.rd=3
    else:
        if "react_up" in diff.func:
            print('in')
            if "react_down"  in diff.func:
                diff.func.pop("react_down")
            mem.dtf=np.nan
            mem.rd=np.nan
        else:
            # there is no responsive switching at all
            mem.thresh=np.nan
            mem.ra=np.nan


def _get_contri_detection_2(defi, s):
    last=defi.pop()
    if s:
        if last=='1':
            c='full response'
        else:
            c='alt response '
    else:
        if last=='1':
            c='no down resp '
        else:
            c='no response  '
    defi.append(c)


_set_contri=[_set_contri_consumption,
            _set_contri_delay,
            _set_contri_switching,
            _set_contri_detection_1,
            _set_contri_detection_2]


_get_contri=[_get_contri_consumption,
            _get_contri_delay,
            _get_contri_switching,
            _get_contri_detection_1,
            _get_contri_detection_2]

blacklist=np.ones(32, dtype=bool)

# no switch and no response -> always 0%
blacklist[0:4]=False

# 24:32 → unstable numerically → don't do that for now
blacklist[24:32]=False

# 9, 11
# co-consume + no down response + no switch -> ara users are just faster
# -> % tends to one, no matter what
blacklist[9]=False
blacklist[11]=False

def set_contri(mem, num):
    """ set the contribution according to num
    return the differential function and the according memory
    """
    diff=Differential()
    print(num)
    num=int(num)
    for f in _set_contri:
        f(diff, mem, num%2)
        num//=2
    print(diff.func)
    return diff.diff, mem


def get_contri(num):
    """ set the contribution according to num
    return the differential function and the according memory
    """
    print(num)
    num=int(num)
    definition=[]
    for f in _get_contri:
        f(definition, num%2)
        num//=2
    print(definition)
    return definition

"""
In here, assemble the different contributions to make a differential equation
"""
def default():
    """ returns a function, is not the function !"""
    return Differential().diff

def no_down():
    """like default but population doesn't respond to down regulation"""
    c2=contrib.copy()
    c2.pop("react_down")
    return Differential(functions=c2).diff

def alt_react_2(mem):
    diff=Differential()
    num=int(19)
    # num=int(18)
    # num=int(17)
    # num=int(16)
    for f in _set_contri[:]:
        f(diff, mem, num%2)
        num//=2
    diff.func["alt_react"]=ctrb.alt_react_2
    print(diff.func)
    return diff.diff

class Differential:
    """
    In here, let compose a differential model from different contributions
    """
    def __init__(self, functions=None):
        if functions is None:
            functions=contrib.copy()
        self.func=functions

    def diff(self, t, x, p):
        # p is the parameters
        x[x<0]=0
        if x[5]>1:
            x[5]=1
        dx=np.zeros_like(x)
        # print(t)
        # print(x)
        for f in self.func.values():
            f(dx, x, t, p)
            # print(f)
            # print(dx)
            # print(p.lock)

        dx[np.all((dx<0,x==0), axis=0)]=0

        # shouldn't be necessary, but you never know:
        if x[5]==1 and dx[5]>0:
            dx[5]=0

        p.store(dx, x, t)

        return dx
