# personal modules
import segmodels
import segdata
import mcmc
from metrowrapper import MetroWrapper

# standalone file
from file_list import l

# standard modules
import numpy as np
import math
import matplotlib.pyplot as plt
import tensorflow as tf
import memory_profiler
import os.path

# remove tf log messages:
import os
os.environ["TF_CPP_MIN_LOG_LEVEL"] = "2"


if __name__ == '__main__':

    # test on the first 20 h of segregostat (l[0]) and the first 30h of
    # chemostat (l[2])
    #data=np.array([segdata.Data(l[2])])
    data=np.array([segdata.Data(l[0]),segdata.Data(l[2])])
    for d in data:
        d.sanitize()
    data[0].truncate(50)
    data[1].truncate(50)


    # memory:
    #             time,Ca,Cg, p, dp
    mem=np.array([[-5, 0, .01, 0, 0],
                  [ 0, 0, .01, 0, 0]])

    memo=segmodels.Memory(memory=mem)
    memo.ra=.1
    # memo.rd=9
    # memo.mu_ma=.7
    # memo.mu_mg=.4
    # memo.Ka=.2
    # memo.Kg=.1
    print(memo.aggregate())

    # memo.dtf=np.nan
    # memo.rd=np.nan
    # model=segmodels.Model(memo, segmodels.differential.no_down())

    # model=segmodels.Model(memo, segmodels.differential.default())

    diff =segmodels.differential.alt_react_2(memo)
    model=segmodels.Model(memo, diff)

    # make the metrowrapper
    wrapper=MetroWrapper(model, data, beta=1000)
    # beta~=-log(0.1)/2e-3 so that upwards diff of 2e-3 has 10% chance yes

    wrapper.set_models()
    i=33
    for j, m in enumerate(wrapper.models):
        plt.figure(j+1)
        m.run(50)
        plt.plot(m.sol[0,:],m.sol[2,:]/np.sum(m.sol[1:4,:], axis=0),
            label=str(i))
        plt.figure(2+j+1)
        plt.plot(m.sol[0,:],m.sol[2,:],
            label=str(i) + "ara")
        plt.plot(m.sol[0,:],m.sol[1,:]+m.sol[3,:],
            label=str(i) + "glu")
        plt.figure(6+j+1)
        plt.plot(m.sol[0,:],m.sol[4,:],
            label=str(i) + " arabinose")
        plt.plot(m.sol[0,:],m.sol[5,:],
            label=str(i) + " glucose")

    for j, d in enumerate(wrapper.data):
        plt.figure(j+1)
        c=[]
        for k in d.flow[:,5]:
            c.append(k.count_proportion())
        plt.plot(d.flow[:,0], c, label="data")
        plt.legend()
        plt.figure(2+j+1)
        plt.legend()
        plt.figure(6+j+1)
        plt.legend()

    plt.show()
